#pairwise log likelihood
##penalized version (bias.reduction=TRUE)
##non-penalized version (bias.reduction=FALSE)
pwlik_mev <- function(z, theta, sets, n, q, bias.reduction=FALSE){

	p <- length(theta)
	RES <- .C("smith_pwllik_dervs",
						plik = double(n), 
						score = double(n*p), 
						hessian = double(p*(p+1)/2), 
						data=as.double(t(z)), 
						s1=as.double(theta[1]), 
						s2=as.double(theta[2]), 
						s3=as.double(theta[3]),  			  
						as.integer(sets[,1]),
						as.integer(sets[,2]),
						as.double(sets[,3]),
						as.double(sets[,4]),
						as.integer(nrow(sets)),
						as.integer(n),
						as.integer(q), 
						as.integer(bias.reduction), NAOK=TRUE)

	if(bias.reduction){

		lik <- sum(RES$plik)
		score <- matrix(RES$score, n)
		J <- crossprod(score)
		H <- matrix(0,p,p)
		diag(H) <- RES$hessian[1:3]
		H[1,2] <- H[2,1] <- RES$hessian[4]
		H[1,3] <- H[3,1] <- RES$hessian[5]
		H[2,3] <- H[3,2] <- RES$hessian[6]		

		tr.adj <- sum(diag(solve(H)%*%J))/2			
		return(lik + tr.adj)

	}else{

		lik <- sum(RES$plik)
		return(lik)

	}
}


#build a regular grid on integers 1:length(vec)
my_grid <- function(vec)
{
	l <- length(vec)
	res <- .C("my_grid",
			as.double(rep(0,l*l)),
			as.double(rep(0,l*l)),
			as.double(vec),
			as.integer(l)
		)
		cbind(res[[1]],res[[2]])
}

#defines which pairs of observations need to be included in the pairwise loglikelihood
my_sets <- function(locations, d, file_name_number)
{
	n <- nrow(locations)
	Res <- .C("my_sets",
				as.double(locations[,1]),
				as.double(locations[,2]),
				as.double(d),
				as.integer(n),
				as.integer(file_name_number)
		  )

	sets <- read.table(as.character(file_name_number))
	on.exit(file.remove(as.character(file_name_number)))
	sets
}

