## Copyright (C) 2020 Ioannis Kosmidis

## Distributed as part of the supplementary material for the manuscript
## "Empirical bias-reducing adjustments to estimating functions"
##
## Authors: Ioannis Kosmidis
## Date: 06 January 2020
## Licence: GPL 2 or greater
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!

library("parallel")
library("dplyr")
library("ggplot2")
library("data.table")

code_path <- "."
path <- "./images/" 

source(file.path(code_path, "negbin_functions.R"))

get_results <- function(path, image_name, n_cores = 2) {
    files <- dir(path, full.names = TRUE)
    files <- files[grep(image_name, files)]
    files <- files[grep("rda", files)]
    all_results <- NULL
    out <- mclapply(files, function(f) {
        cat("Reading", f, "\n")
        load(f)
        cres <- results
        cres$n <- attr(cres, "n")
        cres$N <- attr(cres, "N")
        cres$replication <- attr(cres, "replication")
        cres
    }, mc.cores = n_cores) 
    as_tibble(rbindlist(out))
}


summarize_results <- function(all_results, threshold = 50) {
    all_results %>% group_by(n, parameter, method) %>%
        mutate(converged = converged & (abs(estimate) < threshold)) %>%
        summarize(n_samples = n(),
                  n_simu = unique(N),
                  perc_converged = 100 * sum(converged) / n_samples,
                  bias = mean(estimate[converged] - true_value[converged]),
                  variance = var(estimate[converged]),
                  mc_error = sd(estimate[converged])/sqrt(n_simu),
                  scaled_bias = bias * unique(n),
                  scaled_bias_mc_error = mc_error * unique(n),
                  max_score = max(abs(score[converged])),
                  n_converged = sum(converged))
}

all_results_mu <- get_results(path, "brquasi-mu-negbin", n_cores = 1)
all_results_musquared <- get_results(path, "brquasi-musquared-negbin", n_cores = 1)

summaries_mu <- summarize_results(all_results_mu)
save(summaries_mu, file = file.path(path, "negbin-summaries_mu.rda"))

summaries_musquared <- summarize_results(all_results_musquared)
save(summaries_musquared, file = file.path(path, "negbin-summaries_musquared.rda"))

